using CoreWebApiWithEntityFramework.WebApi.Controllers;
using CoreWebApiWithEntityFramework.WebApi.Data;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Xunit;

namespace CoreWebApiWithEntityFramework.Tests
{
    public class ProductsControllerTest
    {
        [Fact]
        public async Task Post()
        {
            // Arrange
            var options = new DbContextOptionsBuilder<ProductsContext>()
                .UseInMemoryDatabase(databaseName: "Post test")
                .Options;
            ProductsContext productsContext = new ProductsContext(options);

            ProductsController controller = new ProductsController(productsContext);

            Product product = new Product()
            {
                Name = "some name",
                Sku = "xyz123"
            };

            // Act
            var actionResult = await controller.Post(product);

            // Assert
            var createdResult = actionResult as CreatedResult;
            Assert.Equal(201, createdResult.StatusCode);

            Assert.Equal(1, productsContext.Products.Count());
        }

        [Fact]
        public async Task Get()
        {
            // Arrange
            var options = new DbContextOptionsBuilder<ProductsContext>()
            .UseInMemoryDatabase(databaseName: "Get test")
            .Options;

            ProductsContext productsContext = new ProductsContext(options);
            Product product01 = new Product { Name = "first", ProductId = 1, Sku = "abc" };
            Product product02 = new Product { Name = "second", ProductId = 2, Sku = "def" };

            productsContext.Products.Add(product01);
            productsContext.Products.Add(product02);
            await productsContext.SaveChangesAsync();
            ProductsController controller = new ProductsController(productsContext);

            // Act
            var actionResult = await controller.Get();

            // Assert
            var okObjectResult = actionResult as OkObjectResult;

            List<Product> products = okObjectResult.Value as List<Product>;

            Assert.Equal(2, products.Count());
        }

        [Fact]
        public async Task GetById()
        {
            // Arrange
            var options = new DbContextOptionsBuilder<ProductsContext>()
                .UseInMemoryDatabase(databaseName: "Get by id test")
                .Options;

            ProductsContext productsContext = new ProductsContext(options);
            Product product01 = new Product { Name = "laptop", ProductId = 11, Sku = "sku01" };
            Product product02 = new Product { Name = "monitor", ProductId = 22, Sku = "sku02" };

            productsContext.Products.Add(product01);
            productsContext.Products.Add(product02);
            await productsContext.SaveChangesAsync();
            ProductsController controller = new ProductsController(productsContext);

            // Act
            var actionResult = await controller.Get(22);

            // Assert
            var okObjectResult = actionResult as OkObjectResult;

            Product product = okObjectResult.Value as Product;

            Assert.Equal(22, product.ProductId);
            Assert.Equal("monitor", product.Name);
            Assert.Equal("sku02", product.Sku);
        }

        [Fact]
        public async Task GetNoneFound()
        {
            // Arrange
            var options = new DbContextOptionsBuilder<ProductsContext>()
                .UseInMemoryDatabase(databaseName: "Get none found")
                .Options;

            ProductsContext productsContext = new ProductsContext(options);
            Product product01 = new Product { Name = "laptop", ProductId = 11, Sku = "sku01" };
            Product product02 = new Product { Name = "monitor", ProductId = 22, Sku = "sku02" };

            productsContext.Products.Add(product01);
            productsContext.Products.Add(product02);
            await productsContext.SaveChangesAsync();
            ProductsController controller = new ProductsController(productsContext);

            // Act
            var actionResult = await controller.Get(999);

            // Assert
            var notFoundResult = actionResult as NotFoundResult;

            Assert.Equal(404, notFoundResult.StatusCode);
        }
    }
}
