﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using CoreWebApiWithEntityFramework.WebApi.Data;
using Microsoft.EntityFrameworkCore;

namespace CoreWebApiWithEntityFramework.WebApi.Controllers
{
    [Route("api/[controller]")]
    public class ProductsController : Controller
    {
        private readonly ProductsContext _productsContext;

        public ProductsController(ProductsContext context)
        {
            _productsContext = context;
        }
        // GET api/values
        [HttpGet]
        public async Task<IActionResult> Get()
        {
            var products = await _productsContext.Products.ToListAsync();

            if (!products.Any())
            {
                return NotFound();
            }
            return Ok(products);
        }

        // GET api/values/5
        [HttpGet("{productId}")]
        public async Task<IActionResult> Get(int productId)
        {
            var product = await _productsContext.Products.SingleOrDefaultAsync(p => p.ProductId == productId);

            if (product == null)
            {
                return NotFound();
            }

            return Ok(product);
        }

        // POST api/values
        [HttpPost]
        public async Task<IActionResult> Post([FromBody]Product product)
        {
            _productsContext.Products.Add(product);

            await _productsContext.SaveChangesAsync();

            return Created("", null);
        }
    }
}
